////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// jtag.h
//
// (c) Lukas Franz Josef (Brain) Schrittwieser
//
// Feb 2006
//
// Published under the terms of the GNU general public license
//
//
// Compiler: Microsoft Visual C++ 6.0
//
// This is the software module for the RS232 to JTAG bridge. The birdge together with this module allows to access
// the registers of the attached JTAG chain. This module is written for the W32 library of Microsoft Visual C++
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// 24. April 2006:
// Some changes in the bridge hardware require a new software module. Note: This software works only with the new
// hardware revision 2 (=com4jtag.lif)
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef __JTAG_H__
#define __JTAG_H__


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//   I N C L U D E S

//#include <windows.h>	// import std operating system functions
#include <string>		// needed for namespace std
#include <stdio.h>
#include <usb.h>


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//   D E F I N E S



#define	USBASP_SHARED_VID   0x16C0  /* VOTI */
#define	USBASP_SHARED_PID   0x05DC  /* Obdev's free shared PID */

#define	USBASP_OLD_VID      0x03EB  /* ATMEL */
#define	USBASP_OLD_PID	    0xC7B4  /* (unoffical) USBasp */


#define USBASP_BLOCKFLAG_FIRST    1
#define USBASP_BLOCKFLAG_LAST     2

#define USBASP_READBLOCKSIZE   200
#define USBASP_WRITEBLOCKSIZE  200

#define USB_ERROR_NOTFOUND  1
#define USB_ERROR_ACCESS    2
#define USB_ERROR_IO        3



#define USBASP_FUNC_CONNECT     	1
#define USBASP_FUNC_DISCONNECT  	2
#define USBASP_FUNC_TRANSMIT    	3
#define USBASP_FUNC_READFLASH   	4
#define USBASP_FUNC_ENABLEPROG  	5
#define USBASP_FUNC_WRITEFLASH  	6
#define USBASP_FUNC_READEEPROM  	7
#define USBASP_FUNC_WRITEEEPROM 	8
#define USBASP_FUNC_SETLONGADDRESS 	9

/* jtag commands */
#define USBASP_JTAG_CONF		0x80		/* connect or disconnect */
#define USBASP_JTAG_GET_STATE	0x81		/* get actual tap state */
#define USBASP_JTAG_READ_BACK	0x82		/* get values from readback buffer */
#define USBASP_JTAG_EXECUTE		0x83		/* executes a command sequence */

/* possible commands for the jtag command sequence */
#define JTAG_CM_SHIFT_DR	0x10
#define JTAG_CM_SHIFT_IR	0x20
#define JTAG_CM_RUN_TEST	0x30
#define JTAG_CM_UPDATE_DR	0x40
#define JTAG_CM_UPDATE_IR	0x50
#define JTAG_CM_RESET_TAP	0x60



////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//   C L A S S   D E F I N I T I O N

class Cjtag
{
	
public:
	// JTAG - interface to higher classes
	
	// This method accesses the data register of the JTAG TAP (test access port).
	// You can use this method for chains of any length, you just have to pass the 
	// number of bits you want to shift through the chain in length.
	// writeData is a pointer to an unsigned char array that holds the data to write
	// into the register. The bit's in writeData are shifted out MSB first => bits are
	// shifted: (writeData[0])&0x80 .. writeData[0]&0x01 ... until length bits are shifted
	// After shifting out the bits there are two possibilities:
	//  - the state machine goes back to run-test/idle state (= data is processed by the chip)
	//  - the state machine stays in shift-dr state (= shifting of data is not finished)
	// If stayInShiftState is false the former is used, otherwise the latter.
	// This is usefull to split up the shifting of hughe amounts of data (e.g. configuration
	// of a FPGA.)
	//
	// Return value: the number of bits written to the register
	int accessDR (unsigned char* writeData, unsigned char* readData, int length, bool stayInShiftState, bool readBack);

	// This method works like accessDR, but it uses the TAP's instruction register
	int accessIR (unsigned char* writeData, unsigned char* readData, int length, bool readBack);

	// This function reads the current state of the TAPs state machine
	//int getTAPstate (void) { return TAPstate; }

	// moves the tap into rti state. if we are already in rti state we move there again (looping with tms=0)
	int runTest (void);

	// ensures the internal buffer is transfered to the device
	int execute ();

	// This function puts the state machine in the TEST-LOGIC-RESET state
	// This resets the TAP
	int resetTAP (void);

	// initialize the class and open usb device
	int init ();

	int getState (void);

	// close the COM port
	void close (void);

	// is the Class ready?
	bool isReady (void) { return ((usbhandle==0)?false:true); }

	// Constructor
	Cjtag (void);

	// Destructor
	~Cjtag (void);

private:

	// attributes

	/* the jtag commands are filled into a buffer and transmitted if 
	it gets full or we have to send it (read back of registers) 
	this saves time (lesser USB accesses) */
	unsigned char *cmdBuffer;
	int cmdIndex;
	const int cmdBufferLen;


	// holds a handle to the programmer
	usb_dev_handle *usbhandle;

	int verbose;

	
	// private methods
private:

	
	// filles a command into the command buffer
	// cmd is the command byte to add
	// data is the optional data field, len the number of bytes in data
	int addCmd (unsigned char cmd, const unsigned char* data, unsigned char len);

	int usbOpenDevice(usb_dev_handle **device, int vendor, char *vendorName, int product, char *productName);

	int usbasp_transmit(unsigned char receive, unsigned char functionid, unsigned char send[4], unsigned char * buffer, int buffersize);

	void printBuffer(unsigned char* buf, int len);
	
};


#endif

// end of file jtag.h
